import { useCallback, useState } from 'react';

import { API_URL } from '~utils/constants';
import type { ReportParams, ReportResult } from '~utils/types';

import { useExtension } from './useExtension';

export function useDomainReport(): UseDomainReportResult {
  const extension = useExtension();
  const [errors, setErrors] = useState<{ readonly [key: string]: string }>({});
  const [isSubmitting, setIsSubmitting] = useState<boolean>(false);

  const report = useCallback(
    async (params: ReportParams): Promise<ReportResult | undefined> => {
      try {
        setIsSubmitting(true);

        const reason = params.reason;
        const url = params.url;
        const errors = validateForm(params);

        if (errors) {
          setErrors(errors);
          setIsSubmitting(false);
          return;
        }

        const userAgent = navigator.userAgent;
        const version = extension.version;
        const body = JSON.stringify({ reason, url, userAgent, version });
        const headers = { 'Cache-Control': 'no-cache', 'Content-type': 'application/json' };
        const requestInit = { body, headers, method: 'POST' };
        const response = await fetch(`${API_URL}/report/`, requestInit);

        if (response.ok) {
          return await response.json();
        }

        if (response.status === 429) {
          throw new Error(chrome.i18n.getMessage('report_rateLimitError'));
        }

        throw new Error(chrome.i18n.getMessage('report_unknownError'));
      } catch (error) {
        if (error instanceof Error) {
          setErrors({ url: error.message });
        }
      } finally {
        setIsSubmitting(false);
      }
    },
    [extension.version]
  );

  return {
    errors,
    isSubmitting,
    report,
  };
}

function validateForm(params: ReportParams): Partial<ReportParams> | undefined {
  const { reason, url } = params;
  let errors: Partial<ReportParams> | undefined = undefined;

  if (!reason || reason.length < 10 || reason.length > 1000) {
    errors = {
      ...(errors ?? {}),
      reason: chrome.i18n.getMessage('report_reasonInputError'),
    };
  }

  try {
    if (/\s/.test(url)) throw new Error();
    new URL(url);
  } catch {
    errors = {
      ...(errors ?? {}),
      url: chrome.i18n.getMessage('report_urlInputError'),
    };
  }

  return errors;
}

export interface UseDomainReportResult {
  readonly errors: { readonly [key: string]: string };
  readonly isSubmitting: boolean;
  readonly report: (params: ReportParams) => Promise<ReportResult | undefined>;
}
