import { type ChangeEvent, type JSX, type MouseEvent, useCallback, useRef, useState } from 'react';

import { useExclusions } from '~hooks/useExclusions';
import { DOMAIN_REG_EXP } from '~utils/domain';
import { storage } from '~utils/storage';
import type { DomainConfig } from '~utils/types';

export default function Options(): JSX.Element {
  const exclusions = useExclusions();
  const fileInputRef = useRef<HTMLInputElement | null>(null);
  const [filterValue, setFilterValue] = useState<string>('');

  const handleAddClick = useCallback(async () => {
    const message = chrome.i18n.getMessage('options_addPrompt');
    const domain = window.prompt(message)?.trim().replace('www.', '');

    if (domain && (DOMAIN_REG_EXP.test(domain) || domain === 'localhost')) {
      const prev = await storage.get<DomainConfig>(domain);

      await storage.set(domain, { ...prev, on: false });
    }
  }, []);

  const handleClearClick = useCallback(async () => {
    for (const domain of exclusions) {
      const prev = await storage.get<DomainConfig>(domain.name);

      if (prev?.issue?.url) await storage.set(domain.name, { ...prev, on: true });
      else await storage.remove(domain.name);
    }
  }, [exclusions]);

  const handleDeleteClick = useCallback(async (event: MouseEvent<HTMLButtonElement>) => {
    const { value: domain } = event.currentTarget.dataset;

    if (domain) {
      const prev = await storage.get<DomainConfig>(domain);

      if (prev?.issue?.url) await storage.set(domain, { ...prev, on: true });
      else await storage.remove(domain);
    }
  }, []);

  const handleExportClick = useCallback(() => {
    const anchor = document.createElement('a');
    const now = new Date();
    const day = now.getDate().toString().padStart(2, '0');
    const month = now.getMonth().toString().padStart(2, '0');
    const year = now.getUTCFullYear();
    const text = exclusions.reduce((prev, curr) => `${prev}\n${curr.name}`, '');
    const defaultTitle = `${year}${month}${day}`;
    const customTitle = window.prompt('Enter a file name', defaultTitle);

    if (customTitle) {
      const blob = new Blob([text], { type: 'octet/stream' });
      const url = window.URL.createObjectURL(blob);

      anchor.href = url;
      anchor.download = `${customTitle || defaultTitle}.cdm`;
      anchor.click();
      window.URL.revokeObjectURL(url);
    }
  }, [exclusions]);

  const handleFileChange = useCallback((event: ChangeEvent<HTMLInputElement>) => {
    const file = event.currentTarget.files?.[0];
    const reader = new FileReader();

    reader.addEventListener('load', async (event) => {
      const input =
        event.currentTarget && 'result' in event.currentTarget
          ? (event.currentTarget.result as string)?.split('\n')
          : [];

      for (const value of input) {
        const domain = value.replace('www.', '').trim();

        if (domain && (DOMAIN_REG_EXP.test(domain) || domain === 'localhost')) {
          const prev = await storage.get<DomainConfig>(domain);

          await storage.set(domain, { ...prev, on: false });
        }
      }
    });

    if (file) {
      event.currentTarget.value = '';
      reader.readAsText(file);
    }
  }, []);

  const handleFilterChange = useCallback((event: ChangeEvent<HTMLInputElement>) => {
    setFilterValue(event.currentTarget.value);
  }, []);

  const handleImportClick = useCallback(() => {
    fileInputRef.current?.click();
  }, []);

  return (
    <>
      <header className="header">
        <div>
          <h1>Cookie Dialog Monster &gt; {chrome.i18n.getMessage('options_exclusionListTitle')}</h1>
        </div>
      </header>
      <main className="main">
        <div className="button-group">
          <button className="button" data-variant="large" onClick={handleAddClick}>
            <span>{chrome.i18n.getMessage('options_addButton')}</span>
            <svg
              fill="none"
              height="14"
              strokeLinecap="round"
              strokeLinejoin="round"
              strokeWidth="2"
              stroke="currentColor"
              viewBox="0 0 24 24"
              width="14"
            >
              <line x1="12" y1="5" x2="12" y2="19" />
              <line x1="5" y1="12" x2="19" y2="12" />
            </svg>
          </button>
          <button className="button" data-variant="large" onClick={handleClearClick}>
            <span>{chrome.i18n.getMessage('options_clearButton')}</span>
            <svg
              aria-hidden="true"
              fill="none"
              height="14"
              strokeLinecap="round"
              strokeLinejoin="round"
              strokeWidth="2"
              stroke="currentColor"
              viewBox="0 0 24 24"
              width="14"
            >
              <polyline points="3 6 5 6 21 6" />
              <path d="M19 6v14a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V6m3 0V4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v2" />
              <line x1="10" y1="11" x2="10" y2="17" />
              <line x1="14" y1="11" x2="14" y2="17" />
            </svg>
          </button>
          <button className="button" data-variant="large" onClick={handleImportClick}>
            <span>{chrome.i18n.getMessage('options_importButton')}</span>
            <svg
              aria-hidden="true"
              fill="none"
              height="14"
              strokeLinecap="round"
              strokeLinejoin="round"
              strokeWidth="2"
              stroke="currentColor"
              viewBox="0 0 24 24"
              width="14"
            >
              <path d="M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4" />
              <polyline points="17 8 12 3 7 8" />
              <line x1="12" y1="3" x2="12" y2="15" />
            </svg>
            <input
              accept=".cdm"
              onChange={handleFileChange}
              ref={fileInputRef}
              style={{ display: 'none' }}
              type="file"
            />
          </button>
          <button className="button" data-variant="large" onClick={handleExportClick}>
            <span>{chrome.i18n.getMessage('options_exportButton')}</span>
            <svg
              aria-hidden="true"
              fill="none"
              height="14"
              strokeLinecap="round"
              strokeLinejoin="round"
              strokeWidth="2"
              stroke="currentColor"
              viewBox="0 0 24 24"
              width="14"
            >
              <path d="M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4" />
              <polyline points="7 10 12 15 17 10" />
              <line x1="12" y1="15" x2="12" y2="3" />
            </svg>
          </button>
        </div>
        <input
          className="input"
          onChange={handleFilterChange}
          placeholder={chrome.i18n.getMessage('options_filterPlaceholder')}
          value={filterValue}
        />
        <ul className="exclusion-list">
          {exclusions.length ? (
            exclusions.map((exclusion) => (
              <li key={exclusion.name}>
                <span>{exclusion.name}</span>
                <button className="button" data-value={exclusion.name} onClick={handleDeleteClick}>
                  <svg
                    aria-hidden="true"
                    fill="none"
                    height="14"
                    strokeLinecap="round"
                    strokeLinejoin="round"
                    strokeWidth="2"
                    stroke="currentColor"
                    viewBox="0 0 24 24"
                    width="14"
                  >
                    <polyline points="3 6 5 6 21 6" />
                    <path d="M19 6v14a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V6m3 0V4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v2" />
                    <line x1="10" y1="11" x2="10" y2="17" />
                    <line x1="14" y1="11" x2="14" y2="17" />
                  </svg>
                </button>
              </li>
            ))
          ) : (
            <li>{chrome.i18n.getMessage('options_empty')}</li>
          )}
        </ul>
      </main>
      <footer className="footer" />
    </>
  );
}
