import { type FormEvent, type JSX, type KeyboardEvent, useCallback, useState } from 'react';

import { useBrowser } from '~hooks/useBrowser';
import { useData } from '~hooks/useData';
import { useDomain } from '~hooks/useDomain';
import { useDomainReport } from '~hooks/useDomainReport';
import { useExtension } from '~hooks/useExtension';
import { CHROME_STORE_URL, EDGE_STORE_URL, FIREFOX_STORE_URL } from '~utils/constants';
import { suppressLastError } from '~utils/error';
import type { ReportParams } from '~utils/types';

export default function Popup(): JSX.Element {
  const { isChromium, isEdge, isFirefox } = useBrowser();
  const { data, isLoading, isRefetched, refetch } = useData();
  const { config, domain, isSupported, toggleStatus } = useDomain();
  const { errors, isSubmitting, report } = useDomainReport();
  const extension = useExtension();
  const [issueExists, setIssueExists] = useState<boolean>(false);
  const [issueURL, setIssueURL] = useState<string>('');
  const [view, setView] = useState<'main' | 'report'>('main');

  const handleCancelClick = useCallback(() => {
    setIssueExists(false);
    setIssueURL('');
    setView('main');
  }, []);

  const handleInputKeyDown = useCallback(
    (event: KeyboardEvent<HTMLInputElement | HTMLTextAreaElement>) => {
      if (event.key === 'Enter') {
        event.preventDefault();
        event.currentTarget.blur();
      }
    },
    []
  );

  const handlePowerClick = useCallback(async () => {
    await toggleStatus();
    window.close();
  }, [toggleStatus]);

  const handleRefreshClick = useCallback(async () => {
    await refetch();
  }, [refetch]);

  const handleReportClick = useCallback(() => {
    setView('report');
  }, []);

  const handleSettingsClick = useCallback(async () => {
    chrome.runtime.openOptionsPage(suppressLastError);
    window.close();
  }, []);

  const handleSubmit = useCallback(
    async (event: FormEvent<HTMLFormElement>) => {
      event.preventDefault();

      const params = Object.fromEntries(Object.entries(new FormData(event.currentTarget)));
      const result = await report(params as ReportParams);

      if (result) {
        setIssueExists(!result.success);
        setIssueURL(result.data);
      }
    },
    [report]
  );

  return (
    <>
      <header className="header">
        <h1>Cookie Dialog Monster</h1>
        <div className="header-actions">
          <a
            className="header-button"
            href="https://git.wanhose.dev/wanhose/cookie-dialog-monster/wiki/Help-or-issues%3F"
            target="_blank"
            rel="noreferrer"
          >
            <svg
              aria-hidden="true"
              fill="none"
              height="18"
              strokeLinecap="round"
              strokeLinejoin="round"
              strokeWidth="2"
              stroke="currentColor"
              viewBox="0 0 24 24"
              width="18"
            >
              <circle cx="12" cy="12" r="10" />
              <path d="M9.09 9a3 3 0 0 1 5.83 1c0 2-3 3-3 3" />
              <line x1="12" y1="17" x2="12.01" y2="17" />
            </svg>
          </a>
          <button className="header-button" onClick={handleSettingsClick}>
            <svg
              aria-hidden="true"
              fill="none"
              height="18"
              strokeLinecap="round"
              strokeLinejoin="round"
              strokeWidth="2"
              stroke="currentColor"
              viewBox="0 0 24 24"
              width="18"
            >
              <circle cx="12" cy="12" r="3" />
              <path d="M19.4 15a1.65 1.65 0 0 0 .33 1.82l.06.06a2 2 0 0 1 0 2.83 2 2 0 0 1-2.83 0l-.06-.06a1.65 1.65 0 0 0-1.82-.33 1.65 1.65 0 0 0-1 1.51V21a2 2 0 0 1-2 2 2 2 0 0 1-2-2v-.09A1.65 1.65 0 0 0 9 19.4a1.65 1.65 0 0 0-1.82.33l-.06.06a2 2 0 0 1-2.83 0 2 2 0 0 1 0-2.83l.06-.06a1.65 1.65 0 0 0 .33-1.82 1.65 1.65 0 0 0-1.51-1H3a2 2 0 0 1-2-2 2 2 0 0 1 2-2h.09A1.65 1.65 0 0 0 4.6 9a1.65 1.65 0 0 0-.33-1.82l-.06-.06a2 2 0 0 1 0-2.83 2 2 0 0 1 2.83 0l.06.06a1.65 1.65 0 0 0 1.82.33H9a1.65 1.65 0 0 0 1-1.51V3a2 2 0 0 1 2-2 2 2 0 0 1 2 2v.09a1.65 1.65 0 0 0 1 1.51 1.65 1.65 0 0 0 1.82-.33l.06-.06a2 2 0 0 1 2.83 0 2 2 0 0 1 0 2.83l-.06.06a1.65 1.65 0 0 0-.33 1.82V9a1.65 1.65 0 0 0 1.51 1H21a2 2 0 0 1 2 2 2 2 0 0 1-2 2h-.09a1.65 1.65 0 0 0-1.51 1z" />
            </svg>
          </button>
        </div>
      </header>
      <main>
        {config.issue?.url ? (
          <p className="banner" data-variant="warning" role="alert">
            <span>
              {chrome.i18n.getMessage(
                config.issue.flags?.includes('wontfix')
                  ? 'popup_bannerIssueWontFix'
                  : 'popup_bannerIssueOpen'
              )}
            </span>
            &nbsp;
            <a href={config.issue.url} target="_blank" rel="noreferrer">
              <svg
                aria-hidden="true"
                fill="none"
                height="12"
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                stroke="currentColor"
                viewBox="0 0 24 24"
                width="12"
              >
                <path d="M18 13v6a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2V8a2 2 0 0 1 2-2h6" />
                <polyline points="15 3 21 3 21 9" />
                <line x1="10" y1="14" x2="21" y2="3" />
              </svg>
            </a>
          </p>
        ) : null}
        {extension.updateAvailable ? (
          <p className="banner" data-variant="notice" role="alert">
            <span>{chrome.i18n.getMessage('popup_bannerUpdateAvailable')}</span>
            &nbsp;
            <a
              href={`https://git.wanhose.dev/wanhose/cookie-dialog-monster/releases/${extension.updateAvailable}`}
              target="_blank"
              rel="noreferrer"
            >
              <svg
                aria-hidden="true"
                fill="none"
                height="12"
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                stroke="currentColor"
                viewBox="0 0 24 24"
                width="12"
              >
                <path d="M18 13v6a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2V8a2 2 0 0 1 2-2h6" />
                <polyline points="15 3 21 3 21 9" />
                <line x1="10" y1="14" x2="21" y2="3" />
              </svg>
            </a>
          </p>
        ) : null}
        {isSupported ? null : (
          <p className="banner" data-variant="error" role="alert">
            <span>{chrome.i18n.getMessage('popup_bannerSupport')}</span>
            &nbsp;
            <a
              href="https://git.wanhose.dev/wanhose/cookie-dialog-monster/wiki/List-of-unsupported-sites"
              target="_blank"
              rel="noreferrer"
            >
              <svg
                aria-hidden="true"
                fill="none"
                height="12"
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                stroke="currentColor"
                viewBox="0 0 24 24"
                width="12"
              >
                <path d="M18 13v6a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2V8a2 2 0 0 1 2-2h6" />
                <polyline points="15 3 21 3 21 9" />
                <line x1="10" y1="14" x2="21" y2="3" />
              </svg>
            </a>
          </p>
        )}
        {view === 'main' ? (
          <div className="content">
            <button
              className="popup-button power-option"
              data-value={config.on ? 'on' : 'off'}
              disabled={!!config.issue || !!extension.updateAvailable || !isSupported}
              onClick={handlePowerClick}
            >
              <svg
                aria-hidden="true"
                fill="none"
                height="32"
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                stroke="currentColor"
                viewBox="0 0 24 24"
                width="32"
              >
                <path d="M18.36 6.64a9 9 0 1 1-12.73 0" />
                <line x1="12" y1="2" x2="12" y2="12" />
              </svg>
              <span>{domain}</span>
            </button>
            <button
              className="popup-button"
              disabled={!!config.issue || !!extension.updateAvailable || !isSupported}
              onClick={handleReportClick}
            >
              <svg
                aria-hidden="true"
                fill="none"
                height="32"
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                stroke="currentColor"
                viewBox="0 0 24 24"
                width="32"
              >
                <path d="M4 15s1-1 4-1 5 2 8 2 4-1 4-1V3s-1 1-4 1-5-2-8-2-4 1-4 1z" />
                <line x1="4" y1="22" x2="4" y2="15" />
              </svg>
              <span>{chrome.i18n.getMessage('contextMenu_reportOption')}</span>
            </button>
            <a
              className="popup-button"
              href="https://git.wanhose.dev/wanhose/cookie-dialog-monster"
              target="_blank"
              rel="noreferrer"
            >
              <svg
                aria-hidden="true"
                fill="none"
                height="32"
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                stroke="currentColor"
                viewBox="0 0 24 24"
                width="32"
              >
                <circle cx="18" cy="18" r="3" />
                <circle cx="6" cy="6" r="3" />
                <path d="M13 6h3a2 2 0 0 1 2 2v7" />
                <line x1="6" y1="9" x2="6" y2="21" />
              </svg>
              <span>{chrome.i18n.getMessage('popup_contributeOption')}</span>
            </a>
            <a
              className="popup-button rate-option"
              href={
                isEdge
                  ? EDGE_STORE_URL
                  : isChromium
                    ? CHROME_STORE_URL
                    : isFirefox
                      ? FIREFOX_STORE_URL
                      : '#'
              }
              target="_blank"
              rel="noreferrer"
            >
              <svg
                aria-hidden="true"
                fill="none"
                height="32"
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                stroke="currentColor"
                viewBox="0 0 24 24"
                width="32"
              >
                <polygon points="12 2 15.09 8.26 22 9.27 17 14.14 18.18 21.02 12 17.77 5.82 21.02 7 14.14 2 9.27 8.91 8.26 12 2" />
              </svg>
              <span>{chrome.i18n.getMessage('popup_rateOption')}</span>
            </a>
            <div className="popup-data-container">
              <div className="popup-data">
                <b>{chrome.i18n.getMessage('popup_databaseVersion')}</b>
                <span>{data?.version || '?'}</span>
                <button
                  className="popup-data-button"
                  data-animation="flip"
                  data-refreshing={isLoading && !isRefetched}
                  data-refreshed={isRefetched}
                  disabled={isLoading || isRefetched}
                  onClick={handleRefreshClick}
                >
                  {isRefetched ? (
                    <svg
                      aria-hidden="true"
                      fill="none"
                      height="12"
                      strokeLinecap="round"
                      strokeLinejoin="round"
                      strokeWidth="2"
                      stroke="var(--color-success)"
                      viewBox="0 0 24 24"
                      width="12"
                    >
                      <polyline points="20 6 9 17 4 12" />
                    </svg>
                  ) : (
                    <svg
                      aria-hidden="true"
                      fill="none"
                      height="12"
                      strokeLinecap="round"
                      strokeLinejoin="round"
                      strokeWidth="2"
                      stroke="currentColor"
                      viewBox="0 0 24 24"
                      width="12"
                    >
                      <polyline points="1 4 1 10 7 10" />
                      <polyline points="23 20 23 14 17 14" />
                      <path d="M20.49 9A9 9 0 0 0 5.64 5.64L1 10m22 4l-4.64 4.36A9 9 0 0 1 3.51 15" />
                    </svg>
                  )}
                </button>
              </div>
              <div className="popup-data">
                <b>{chrome.i18n.getMessage('popup_extensionVersion')}</b>
                <span>{extension.version}</span>
              </div>
            </div>
          </div>
        ) : (
          <div className="report">
            {issueURL ? (
              <>
                {issueExists ? (
                  <div className="report-submit-error-view" hidden>
                    <svg
                      aria-hidden="true"
                      fill="none"
                      height="48"
                      strokeLinecap="round"
                      strokeLinejoin="round"
                      strokeWidth="2"
                      stroke="var(--color-error)"
                      viewBox="0 0 24 24"
                      width="48"
                    >
                      <circle cx="12" cy="12" r="10" />
                      <line x1="12" y1="8" x2="12" y2="12" />
                      <line x1="12" y1="16" x2="12.01" y2="16" />
                    </svg>
                    <div className="report-submit-text">
                      {chrome.i18n.getMessage('report_submitErrorText')}
                    </div>
                    <div className="report-submit-extra-text">
                      {chrome.i18n.getMessage('report_submitErrorExtraText')}
                    </div>
                    <div className="report-issue-button">
                      {chrome.i18n.getMessage('contextMenu_issueOption')}
                    </div>
                  </div>
                ) : (
                  <div className="report-submit-success-view" hidden>
                    <svg
                      aria-hidden="true"
                      fill="none"
                      height="48"
                      strokeLinecap="round"
                      strokeLinejoin="round"
                      strokeWidth="2"
                      stroke="var(--color-success)"
                      viewBox="0 0 24 24"
                      width="48"
                    >
                      <path d="M22 11.08V12a10 10 0 1 1-5.93-9.14" />
                      <polyline points="22 4 12 14.01 9 11.01" />
                    </svg>
                    <div className="report-submit-text">
                      {chrome.i18n.getMessage('report_submitSuccessText')}
                    </div>
                    <div className="report-submit-extra-text">
                      {chrome.i18n.getMessage('report_submitSuccessExtraText')}
                    </div>
                    <div className="report-issue-button">
                      {chrome.i18n.getMessage('contextMenu_issueOption')}
                    </div>
                  </div>
                )}
              </>
            ) : (
              <div className="report-form-view">
                <div className="report-body-text">{chrome.i18n.getMessage('report_bodyText')}</div>
                <form className="report-form" noValidate onSubmit={handleSubmit}>
                  <div className="report-input-group">
                    <div className="report-input-label" id="report-label-url">
                      <span>{chrome.i18n.getMessage('report_urlInputLabel')}</span>
                      <span className="report-input-label-required">*</span>
                    </div>
                    <input
                      aria-invalid={!!errors['url']}
                      aria-labelledby="report-label-url"
                      aria-required="true"
                      className="report-input"
                      disabled={isSubmitting}
                      name="url"
                      onKeyDown={handleInputKeyDown}
                      placeholder="https://www.example.com/"
                    />
                    <div aria-hidden={!errors['url']} className="report-input-error">
                      {chrome.i18n.getMessage('report_urlInputError')}
                    </div>
                  </div>
                  <div className="report-input-group">
                    <div className="report-input-label" id="report-label-reason">
                      <span>{chrome.i18n.getMessage('report_reasonInputLabel')}</span>
                      <span className="report-input-label-required">*</span>
                    </div>
                    <textarea
                      aria-invalid={!!errors['reason']}
                      aria-labelledby="report-label-reason"
                      aria-required="true"
                      className="report-input"
                      disabled={isSubmitting}
                      name="reason"
                      onKeyDown={handleInputKeyDown}
                      placeholder={chrome.i18n.getMessage('report_reasonInputPlaceholder')}
                      rows={4}
                    />
                    <div aria-hidden={!errors['reason']} className="report-input-error">
                      {chrome.i18n.getMessage('report_reasonInputError')}
                    </div>
                  </div>
                  <div className="report-buttons">
                    <button className="report-submit-button" disabled={isSubmitting} type="submit">
                      {chrome.i18n.getMessage('contextMenu_reportOption')}
                    </button>
                    <button
                      className="report-cancel-button"
                      disabled={isSubmitting}
                      onClick={handleCancelClick}
                    >
                      {chrome.i18n.getMessage('report_cancelButtonText')}
                    </button>
                  </div>
                </form>
              </div>
            )}
          </div>
        )}
      </main>
      <footer />
    </>
  );
}
